#include "Arduino.h"
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"
#define MAXVOLUME 25
//#define volumeControl

SoftwareSerial softwareSerial1(10, 11); // RX, TX
SoftwareSerial softwareSerial2(2, 3); // RX, TX
DFRobotDFPlayerMini Player1;
DFRobotDFPlayerMini Player2;

struct player {
  String name;
  int button1pin;
  int button2pin;
  int potPin;
  bool button1State;
  bool button2State;
  int volume;
  SoftwareSerial* serial;
  DFRobotDFPlayerMini &player;
};

player player1 = {"player1", 4, 5, A0, HIGH, HIGH, 0, &softwareSerial1, Player1};
player player2 = {"player2", 6, 7, A1, HIGH, HIGH, 0, &softwareSerial2, Player2};


void printDetail(uint8_t type, int value);

void setup()
{
  Serial.begin(115200);
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, LOW);
  initialise(player1);
  delay(2000);
  initialise(player2);
  delay(2000);
  digitalWrite(LED_BUILTIN, HIGH);
}

void loop() {
  DFRobotDFPlayerMini *test = &player1.player;
  playerHandle(&player1);
  playerHandle(&player2);
  delay(100);
}

void initialise(player DFplayer) {
  pinMode(DFplayer.button1pin, INPUT_PULLUP);
  pinMode(DFplayer.button2pin, INPUT_PULLUP);
  DFplayer.serial->begin(9600);
  if (!DFplayer.player.begin(*DFplayer.serial)) {  //Use softwareSerial to communicate with mp3.
    Serial.println(F("Unable to begin:"));
    Serial.println(F("1.Please recheck the connection!"));
    Serial.println(F("2.Please insert the SD card!"));
    while (true) { // blink fast on initialisation error
      digitalWrite(LED_BUILTIN, HIGH);
      delay(300);
      digitalWrite(LED_BUILTIN, LOW);
      delay(300);
      }
  }
  delay(1000);
  Serial.println(F("DFPlayer Mini online."));
  DFplayer.player.setTimeOut(500); //Set serial communication time out 500ms
  //----Set volume----
  DFplayer.player.volume(MAXVOLUME);  //Set volume value (0~30).
  //----Set different EQ----
    DFplayer.player.EQ(DFPLAYER_EQ_BASS);
  //----Set device we use SD as default----
  DFplayer.player.outputDevice(DFPLAYER_DEVICE_SD);
  //----Mp3 control----
  DFplayer.player.enableDAC();  //Enable On-chip DAC
}


void playerHandle (player *DFplayer) {
  bool button1 = digitalRead(DFplayer->button1pin);
  bool button2 = digitalRead(DFplayer->button2pin);
  int potValue = analogRead(DFplayer->potPin);
  DFRobotDFPlayerMini Player = DFplayer->player;
  if (button1 == LOW && DFplayer->button1State == HIGH) Player.play(1);
  if (button2 == LOW && DFplayer->button2State == HIGH) Player.play(2);
  #ifdef volumeControl
  int volume = map(potValue, 0, 1023, 0, MAXVOLUME);
  if (volume != DFplayer->volume) {Player.volume(volume); Serial.println(volume);}
  #endif
  DFplayer->button1State = button1;
  DFplayer->button2State = button2;
  DFplayer->volume = volume;
}

void printDetail(uint8_t type, int value) {
  switch (type) {
    case TimeOut:
      Serial.println(F("Time Out!"));
      break;
    case WrongStack:
      Serial.println(F("Stack Wrong!"));
      break;
    case DFPlayerCardInserted:
      Serial.println(F("Card Inserted!"));
      break;
    case DFPlayerCardRemoved:
      Serial.println(F("Card Removed!"));
      break;
    case DFPlayerCardOnline:
      Serial.println(F("Card Online!"));
      break;
    case DFPlayerUSBInserted:
      Serial.println("USB Inserted!");
      break;
    case DFPlayerUSBRemoved:
      Serial.println("USB Removed!");
      break;
    case DFPlayerPlayFinished:
      Serial.print(F("Number:"));
      Serial.print(value);
      Serial.println(F(" Play Finished!"));
      break;
    case DFPlayerError:
      Serial.print(F("DFPlayerError:"));
      switch (value) {
        case Busy:
          Serial.println(F("Card not found"));
          break;
        case Sleeping:
          Serial.println(F("Sleeping"));
          break;
        case SerialWrongStack:
          Serial.println(F("Get Wrong Stack"));
          break;
        case CheckSumNotMatch:
          Serial.println(F("Check Sum Not Match"));
          break;
        case FileIndexOut:
          Serial.println(F("File Index Out of Bound"));
          break;
        case FileMismatch:
          Serial.println(F("Cannot Find File"));
          break;
        case Advertise:
          Serial.println(F("In Advertise"));
          break;
        default:
          break;
      }
      break;
    default:
      break;
  }
}
